.g-user-list-entry
  padding 11px 14px
  margin 2px 0
  border-radius 3px
  background-color #fafafc
  border 1px solid #eaeaea
  box-shadow inset 0 0 2px 2px #f4f4f4

  .g-user-title
    font-size 17px

  .g-user-login
    color #666
    font-size 16px
    font-weight bold

  .g-space-used
    margin-top 2px

  .g-user-status
    margin-top 2px
    color #d9534f

  .g-user-right-column
    border-left 1px solid #ddd
    padding-left 8px
    display inline
    width 235px
    font-size 14px
    float right
    color #909090

  .g-user-joined-date, .g-user-space-used
    color #444

.g-user-list-header
  padding-bottom 10px

  .g-user-pagination
    display inline-block
    >ul
      margin 0 13px 0 0

    .pagination>li>a
      float none
      line-height 1.5
      padding 7px 12px

  .g-user-sort
    display inline-block
    margin-right 12px

  .g-user-search-form
    display inline
    .form-group
      display inline-block

    input[type="text"]
      width 170px
      margin-right 13px

  .g-user-count-container
    display inline-block
    position relative

    i
      color #888
      font-size 16px
      vertical-align middle

    .g-user-count
      background-color #eee
      border-radius 3px
      padding 0 0.2em
      font-size 9px
      position absolute
      left 72%
      top 55%
